@@ -15,6 +15,7 @@ sub options {
     ('m|message=s'  => 'message',
      'C|check-only' => 'check_only',
      'S|sign'	  => 'sign',
+     'P|patch=s'  => 'patch',
      'import',	  => 'import',
      'direct',	  => 'direct',
     );
@@ -33,18 +34,21 @@ sub target_prompt { $target_prompt }
 
 sub under_mirror {
     my ($self, $target) = @_;
+    return if $self->{direct};
     HAS_SVN_MIRROR and SVN::Mirror::is_mirrored ($target->{repos}, $target->{path});
 }
 
 sub check_mirrored_path {
     my ($self, $target) = @_;
-    if (!$self->{direct} && $self->under_mirror ($target)) {
+    if ($self->under_mirror ($target)) {
 	print loc ("%1 is under mirrored path.\n", $target->{depotpath});
 	return;
     }
     return 1;
 }
 
+# OBSOLETED: do not create new callers to this function.
+# Use get_dynamic_editor instead.
 sub get_commit_editor {
     my ($self, $xdroot, $committed, $path, %arg) = @_;
     ${$arg{callback}} = $committed if $arg{editor};
@@ -69,10 +73,34 @@ sub get_commit_message {
 	(loc('log message'), $target_prompt, join ("\n", $msg || '', $target_prompt, ''), 'commit');
 }
 
+# XXX: This should just return Editor::Dynamic objects
+sub get_dynamic_editor {
+    my ($self, $target) = @_;
+    my $m = $self->under_mirror ($target);
+    my $anchor = $m ? $m->{target_path} : '/';
+    my ($storage, %cb) = $self->get_editor ($target->new (path => $anchor));
+    my $editor = SVK::Editor::Rename->new
+	( editor => $storage,
+	  cb_exist => $self->{parent} ? $cb{cb_exist} : undef);
+    $editor->{_root_baton} = $editor->open_root ($cb{cb_rev}->(''));
+    return ($anchor, $editor);
+}
+
+sub finalize_dynamic_editor {
+    my ($self, $editor) = @_;
+    $editor->close_directory ($editor->{_root_baton});
+    $editor->close_edit;
+}
+
+sub adjust_anchor {
+    my ($self, $editor) = @_;
+    $editor->adjust_anchor ($editor->{edit_tree}[0][-1]);
+}
+
 # Return the editor according to copath, path, and is_mirror (path)
 # It will be Editor::XD, repos_commit_editor, or svn::mirror merge back editor.
 sub get_editor {
-    my ($self, $target, $callback) = @_;
+    my ($self, $target, $callback, $source) = @_;
     my ($editor, %cb);
 
     # XXX: the case that the target is an xd is actually only used in merge.
@@ -94,11 +122,15 @@ sub get_editor {
     my ($base_rev, $m, $mpath);
     if (!$self->{direct} and HAS_SVN_MIRROR and
 	(($m, $mpath) = SVN::Mirror::is_mirrored ($target->{repos}, $target->{path}))) {
-	print loc("Merging back to SVN::Mirror source %1.\n", $m->{source});
-	if ($self->{check_only}) {
-	    print loc("Checking against mirrored directory locally.\n");
+	if ($self->{patch}) {
+	    print loc("Patching locally against mirror source %1.\n", $m->{source});
+	    $base_rev = $m->{fromrev};
+	}
+	elsif ($self->{check_only}) {
+	    print loc("Checking locally against mirror source %1.\n", $m->{source});
 	}
 	else {
+	    print loc("Merging back to mirror source %1.\n", $m->{source});
 	    $m->{config} = $self->{svnconfig};
 	    $m->{revprop} = ['svk:signature'];
 	    ($base_rev, $editor) = $m->get_merge_back_editor
@@ -119,6 +151,27 @@ sub get_editor {
     my $yrev = $fs->youngest_rev;
     my $root = $fs->revision_root ($yrev);
 
+    %cb = SVK::Editor::Merge::cb_for_root
+	($root, $target->{path}, defined $base_rev ? $base_rev : $yrev);
+
+    if ($self->{patch}) {
+	require SVK::Patch;
+	die loc ("Illegal patch name: %1.\n", $self->{patch})
+	    if $self->{patch} !~ m/^[\w\-]+$/;
+	my $patch = SVK::Patch->new ($self->{patch}, $self->{xd},
+				     $target->depotname, $source, $target);
+	$patch->{ticket} = SVK::Merge->new (xd => $self->{xd})->merge_info ($source)->add_target ($source)->as_string
+	    if $source;
+	$patch->{log} = $self->{message};
+	my $fname = $self->{xd}->patch_file ($self->{patch});
+	if (-e $fname) {
+	    die loc ("file %1 already exists.\n", $fname).
+		($source ? loc ("use 'svk patch regen %1' instead.\n", $self->{patch}) : '');
+	}
+	return ($patch->commit_editor ($fname), %cb, callback => \$callback,
+		send_fulltext => 0);
+    }
+
     $editor ||= $self->{check_only} ? SVN::Delta::Editor->new :
 	SVN::Delta::Editor->new
 	( $target->{repos}->get_commit_editor
@@ -138,9 +191,6 @@ sub get_editor {
 							       );
     }
 
-    %cb = SVK::Editor::Merge::cb_for_root
-	($root, $target->{path}, defined $base_rev ? $base_rev : $yrev);
-
     unless ($self->{check_only}) {
 	for ($SVN::Error::FS_TXN_OUT_OF_DATE, $SVN::Error::FS_ALREADY_EXISTS) {
 	    # XXX: there's no copath info here
@@ -150,7 +200,8 @@ sub get_editor {
 	}
     }
 
-    return ($editor, %cb, mirror => $m, callback => \$callback);
+    return ($editor, %cb, mirror => $m, callback => \$callback,
+	    send_fulltext => !$cb{mirror});
 }
 
 
@@ -252,10 +303,10 @@ sub committed_commit {
 	    $path = '' if $path eq '/';
 	    my $dpath = abs2rel($copath, $target->{copath} => $path, '/');
 	    my $prop = $root->node_proplist ($dpath);
-	    # XXX: some mode in get_fh for modification only
 	    my $layer = SVK::XD::get_keyword_layer ($root, $dpath, $prop);
-	    my $eol = SVK::XD::get_eol_layer ($root, $dpath, $prop);
-	    next unless $layer || ($eol ne ':raw' && $eol ne '');
+	    my $eol = SVK::XD::get_eol_layer ($root, $dpath, $prop, '>');
+	    # XXX: can't bypass eol translation when normalization needed
+	    next unless $layer || ($eol ne ':raw' && $eol ne ' ');
 
 	    my $fh = $root->file_contents ($dpath);
 	    my $perm = (stat ($copath))[2];
@@ -281,7 +332,7 @@ sub run {
 
     my $is_mirrored = $self->under_mirror ($target) && !$self->{direct};
     print loc("Commit into mirrored path: merging back directly.\n")
-	if $is_mirrored;
+	if $is_mirrored and !$self->{patch};
 
     # XXX: should use some status editor to get the committed list for post-commit handling
     # while printing the modified nodes.
@@ -298,7 +349,7 @@ sub run {
     my ($editor, %cb) = $self->get_editor ($target->new (copath => undef), $committed);
 
     die loc("unexpected error: commit to mirrored path but no mirror object")
-	if $is_mirrored && !$self->{direct} && !$cb{mirror};
+	if $is_mirrored and !($self->{direct} or $self->{patch} or $cb{mirror});
 
     $self->run_delta ($target, $xdroot, $editor, %cb);
 }
@@ -312,13 +363,15 @@ sub run_delta {
 	  error => $self->{error},
 	  xdroot => $xdroot,
 	  editor => $editor,
+	  send_delta => !$cb{send_fulltext},
+	  nodelay => $cb{send_fulltext},
 	  $self->{import} ?
 	  ( auto_add => 1,
+	    obstruct_as_replace => 1,
 	    absent_as_delete => 1) :
 	  ( absent_ignore => 1),
 	  $cb{mirror} ?
-	  ( send_delta => 1,
-	    cb_copyfrom => sub {
+	  ( cb_copyfrom => sub {
 		my ($path, $rev) = @_;
 		$path =~ s|^\Q$cb{mirror}{target_path}\E|$cb{mirror}{source}|;
 		return ($path, scalar $cb{mirror}->find_remote_rev ($rev));
@@ -331,8 +384,7 @@ sub run_delta {
 		return $revcache{$corev} if exists $revcache{corev};
 		my $rev = ($fs->revision_root ($corev)->node_history ($revtarget)->prev (0)->location)[1];
 		$revcache{$corev} = $cb{mirror}->find_remote_rev ($rev);
-	    }) :
-	  ( nodelay => 1 ));
+	    }) : ());
     return;
 }
 
