@@ -6,18 +6,33 @@ use base qw( SVK::Command::Update );
 use SVK::XD;
 
 sub options {
-    ('a|all'		=> 'all',
-     'l|lump'		=> 'lump');
+   ('a|all'		=> 'all',
+    'l|lump'		=> 'lump');
 }
 
 sub parse_arg {
     my ($self, @arg) = @_;
 
     @arg = ('') if $#arg < 0;
+
+    if ( @arg == 1 and !$self->arg_co_maybe($arg[0])->{'copath'}) {
+        # If the last argument is a depot path, rather than a copath
+        # then we should do a merge to the local depot, rather than 
+        # an update to the path
+        return $self->rebless (
+            smerge => {
+                to => 1,
+                log => 1,
+                message => '',
+                incremental => !$self->{lump},
+            }
+        )->parse_arg (@arg);
+    }
+
     if ($self->{all}) {
         my $checkout = $self->{xd}{checkout}{hash};
         @arg = sort grep $checkout->{$_}{depotpath}, keys %$checkout;
-    }
+    } 
 
     $self->{lump} = 1; # -- XXX -- will break otherwise -- XXX ---
 
@@ -40,11 +55,18 @@ SVK::Command::Pull - Bring changes from another repository
 
  pull [PATH...]
 
+    Update your local branch and checkout path from the remote
+    master repository.
+
+ pull DEPOTPATH
+
+    Update your local branch from the remote master repository.
+
 =head1 OPTIONS
 
  -a [--all]             : pull into all checkout paths
  -l [--lump]            : merge everything into a single commit log
-                          (always enabled by default in this version)
+                          (always enabled for 'pull PATH' for now)
 
 =head1 AUTHORS
 
